package github.benjamin.equipreservebackend.utils;

import github.benjamin.equipreservebackend.security.SecurityUser;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.crypto.SecretKey;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.concurrent.TimeUnit;

@Component
public class JwtUtil {

    @Value("${jwt.secret}")
    private String SECRET_KEY;

    public String generateToken(SecurityUser securityUser) {
        return Jwts.builder()
                .subject(securityUser.getUser().getId().toString())
                .claim("role", securityUser.getRoles())
                .issuedAt(new Date())
                .expiration(new Date(System.currentTimeMillis() + 3600_000))
                .signWith(getKey(), Jwts.SIG.HS256)
                .compact();
    }

    public Long getUserId(String token) {
        return Long.valueOf(parseToken(token).getSubject());
    }

    public Claims parseToken(String token) {
        return Jwts.parser()
                .verifyWith(getKey())
                .build()
                .parseSignedClaims(token)
                .getPayload();

    }

    /**
     * 如果token在十分钟内会过期则需要刷新
     * @param token
     * @return 是否需要刷新token
     */
    public Boolean shouldRefresh(String token) {
        Date expiration = getExpiration(token);
        Long currentTime = System.currentTimeMillis();
        Long expireTime = expiration.getTime();
        return expireTime - currentTime < TimeUnit.MINUTES.toMillis(10);
    }

    public Date getExpiration(String token) {
        return Jwts.parser()
                .verifyWith(getKey())
                .build()
                .parseSignedClaims(token)
                .getPayload()
                .getExpiration();

    }

    private SecretKey getKey() {
        return Keys.hmacShaKeyFor(SECRET_KEY.getBytes(StandardCharsets.UTF_8));
    }
}
